package sun.nio.fs;

import java.nio.file.attribute.DosFileAttributes;
import java.util.Map;
import java.util.Set;
import java.nio.file.attribute.DosFileAttributeView;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileTime;
import java.io.IOException;

class WindowsFileAttributeViews
{
    WindowsFileAttributeViews() {
    }
    
    static WindowsFileAttributeViews.Basic createBasicView(WindowsPath windowsPath, boolean b) {
        return new WindowsFileAttributeViews.Basic(windowsPath, b);
    }
    
    static WindowsFileAttributeViews.Dos createDosView(WindowsPath windowsPath, boolean b) {
        return new WindowsFileAttributeViews.Dos(windowsPath, b);
    }
    
    private static class Basic extends AbstractBasicFileAttributeView
    {
        final WindowsPath file;
        final boolean followLinks;
        
        Basic(WindowsPath file, boolean followLinks) {
            this.file = file;
            this.followLinks = followLinks;
        }
        
        @Override
        public WindowsFileAttributes readAttributes() throws IOException {
            this.file.checkRead();
            try {
                return WindowsFileAttributes.get(this.file, this.followLinks);
            }
            catch (WindowsException ex) {
                ex.rethrowAsIOException(this.file);
                return null;
            }
        }
        
        private long adjustForFatEpoch(long n) {
            if (n != -1L && n < 119600064000000000L) {
                return 119600064000000000L;
            }
            return n;
        }
        
        void setFileTimes(long n, long n2, long n3) throws IOException {
            long createFile = -1L;
            try {
                int n4 = 33554432;
                if (!this.followLinks && this.file.getFileSystem().supportsLinks()) {
                    n4 |= 0x200000;
                }
                createFile = WindowsNativeDispatcher.CreateFile(this.file.getPathForWin32Calls(), 256, 7, 3, n4);
            }
            catch (WindowsException ex) {
                ex.rethrowAsIOException(this.file);
            }
            try {
                WindowsNativeDispatcher.SetFileTime(createFile, n, n2, n3);
            }
            catch (WindowsException ex2) {
                if (this.followLinks && ex2.lastError() == 87) {
                    try {
                        if (WindowsFileStore.create(this.file).type().equals("FAT")) {
                            WindowsNativeDispatcher.SetFileTime(createFile, this.adjustForFatEpoch(n), this.adjustForFatEpoch(n2), this.adjustForFatEpoch(n3));
                            ex2 = null;
                        }
                    }
                    catch (SecurityException ex3) {}
                    catch (WindowsException ex4) {}
                    catch (IOException ex5) {}
                }
                if (ex2 != null) {
                    ex2.rethrowAsIOException(this.file);
                }
            }
            finally {
                WindowsNativeDispatcher.CloseHandle(createFile);
            }
        }
        
        @Override
        public void setTimes(FileTime fileTime, FileTime fileTime2, FileTime fileTime3) throws IOException {
            if (fileTime == null && fileTime2 == null && fileTime3 == null) {
                return;
            }
            this.file.checkWrite();
            this.setFileTimes((fileTime3 == null) ? -1L : WindowsFileAttributes.toWindowsTime(fileTime3), (fileTime2 == null) ? -1L : WindowsFileAttributes.toWindowsTime(fileTime2), (fileTime == null) ? -1L : WindowsFileAttributes.toWindowsTime(fileTime));
            return;
        }
    }
    
    static class Dos extends WindowsFileAttributeViews.Basic implements DosFileAttributeView
    {
        private static final String READONLY_NAME = "readonly";
        private static final String ARCHIVE_NAME = "archive";
        private static final String SYSTEM_NAME = "system";
        private static final String HIDDEN_NAME = "hidden";
        private static final String ATTRIBUTES_NAME = "attributes";
        static final Set<String> dosAttributeNames;
        
        Dos(WindowsPath windowsPath, boolean b) {
            super(windowsPath, b);
        }
        
        @Override
        public String name() {
            return "dos";
        }
        
        @Override
        public void setAttribute(String s, Object o) throws IOException {
            if (s.equals("readonly")) {
                this.setReadOnly((boolean)o);
                return;
            }
            if (s.equals("archive")) {
                this.setArchive((boolean)o);
                return;
            }
            if (s.equals("system")) {
                this.setSystem((boolean)o);
                return;
            }
            if (s.equals("hidden")) {
                this.setHidden((boolean)o);
                return;
            }
            super.setAttribute(s, o);
            return;
        }
        
        @Override
        public Map<String, Object> readAttributes(String[] array) throws IOException {
            AbstractBasicFileAttributeView.AttributesBuilder create = AbstractBasicFileAttributeView.AttributesBuilder.create(WindowsFileAttributeViews.Dos.dosAttributeNames, array);
            WindowsFileAttributes attributes = super.readAttributes();
            this.addRequestedBasicAttributes(attributes, create);
            if (create.match("readonly")) {
                create.add("readonly", Boolean.valueOf(attributes.isReadOnly()));
            }
            if (create.match("archive")) {
                create.add("archive", Boolean.valueOf(attributes.isArchive()));
            }
            if (create.match("system")) {
                create.add("system", Boolean.valueOf(attributes.isSystem()));
            }
            if (create.match("hidden")) {
                create.add("hidden", Boolean.valueOf(attributes.isHidden()));
            }
            if (create.match("attributes")) {
                create.add("attributes", Integer.valueOf(attributes.attributes()));
            }
            return create.unmodifiableMap();
        }
        
        private void updateAttributes(int n, boolean b) throws IOException {
            this.file.checkWrite();
            String finalPath = WindowsLinkSupport.getFinalPath(this.file, this.followLinks);
            try {
                int getFileAttributes = WindowsNativeDispatcher.GetFileAttributes(finalPath);
                int n2;
                if (b) {
                    n2 = (getFileAttributes | n);
                }
                else {
                    n2 = (getFileAttributes & ~n);
                }
                if (n2 != getFileAttributes) {
                    WindowsNativeDispatcher.SetFileAttributes(finalPath, n2);
                }
            }
            catch (WindowsException ex) {
                ex.rethrowAsIOException(this.file);
            }
        }
        
        @Override
        public void setReadOnly(boolean b) throws IOException {
            this.updateAttributes(1, b);
        }
        
        @Override
        public void setHidden(boolean b) throws IOException {
            this.updateAttributes(2, b);
        }
        
        @Override
        public void setArchive(boolean b) throws IOException {
            this.updateAttributes(32, b);
        }
        
        @Override
        public void setSystem(boolean b) throws IOException {
            this.updateAttributes(4, b);
        }
        
        void setAttributes(WindowsFileAttributes windowsFileAttributes) throws IOException {
            int n = 0;
            if (windowsFileAttributes.isReadOnly()) {
                n |= 0x1;
            }
            if (windowsFileAttributes.isHidden()) {
                n |= 0x2;
            }
            if (windowsFileAttributes.isArchive()) {
                n |= 0x20;
            }
            if (windowsFileAttributes.isSystem()) {
                n |= 0x4;
            }
            this.updateAttributes(n, true);
            this.setFileTimes(WindowsFileAttributes.toWindowsTime(windowsFileAttributes.creationTime()), WindowsFileAttributes.toWindowsTime(windowsFileAttributes.lastModifiedTime()), WindowsFileAttributes.toWindowsTime(windowsFileAttributes.lastAccessTime()));
        }
        
        static {
            dosAttributeNames = Util.newSet(WindowsFileAttributeViews.Dos.basicAttributeNames, new String[] { "readonly", "archive", "system", "hidden", "attributes" });
        }
    }
}
